@@ -390,7 +390,7 @@ ephy_actions_extension_editor_dialog_constructor
 		 G_CALLBACK(ephy_actions_extension_editor_dialog_edit_selected));
 
 	ephy_dialog_construct (EPHY_DIALOG (dialog), properties,
-			       SHARE_DIR "/glade/actions-editor.glade",
+			       SHARE_DIR "/ui/actions-editor.ui",
 			       properties[PROP_ACTIONS_EDITOR].id,
 			       GETTEXT_PACKAGE);
 
@@ -729,26 +729,26 @@ ephy_actions_extension_editor_dialog_response_cb
 	GdkScreen *screen;
 	GError *error = NULL;
 
-	if (response != GTK_RESPONSE_HELP)
-		return;
-
-	screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
-	gtk_show_uri (screen, "ghelp:epiphany-extensions?epi-ext-action-manage",
-		      gtk_get_current_event_time (), &error);
-
-	if (error)
+	if (response == GTK_RESPONSE_HELP)
 	{
-		GtkWidget *errord;
-		errord = gtk_message_dialog_new (GTK_WINDOW (dialog),
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("Could not display help: %s"),
-						 error->message);
-		g_error_free (error);
-		g_signal_connect (errord, "response",
-				  G_CALLBACK (gtk_widget_destroy), NULL);
-		gtk_widget_show (errord);
+		screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
+		gtk_show_uri (screen, "ghelp:epiphany-extensions?epi-ext-action-manage",
+			      gtk_get_current_event_time (), &error);
+
+		if (error)
+		{
+			GtkWidget *errord;
+			errord = gtk_message_dialog_new (GTK_WINDOW (dialog),
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_OK,
+							 _("Could not display help: %s"),
+							 error->message);
+			g_error_free (error);
+			g_signal_connect (errord, "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
+			gtk_widget_show (errord);
+		}
 	}
 
 	g_object_unref (pdialog);
