@@ -1,5 +1,6 @@
 /*
  *  Copyright © 2005 Raphaël Slinckx <raphael@slinckx.net>
+ *  Copyright © 2010 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -92,7 +93,7 @@ static const GtkActionEntry action_entries [] =
 
 /* We got an rss feed from a tab */
 static void
-ephy_rss_ge_feed_cb (EphyEmbed *embed,
+ephy_rss_ge_feed_cb (EphyWebView *view,
 		     const char *type,
 		     const char *title,
 		     const char *address,
@@ -100,9 +101,9 @@ ephy_rss_ge_feed_cb (EphyEmbed *embed,
 {
 	FeedList *list;
 
-	list = (FeedList *) g_object_steal_data (G_OBJECT (embed), FEEDLIST_DATA_KEY);
+	list = (FeedList *) g_object_steal_data (G_OBJECT (view), FEEDLIST_DATA_KEY);
 	list = rss_feedlist_add (list, type, title, address);
-	g_object_set_data_full (G_OBJECT (embed), FEEDLIST_DATA_KEY, list,
+	g_object_set_data_full (G_OBJECT (view), FEEDLIST_DATA_KEY, list,
 				(GDestroyNotify) rss_feedlist_free);
 
 	LOG ("Got a new feed for the site: type=%s, title=%s, address=%s\nWe now have %d feeds", type, title, address, rss_feedlist_length (list));
@@ -114,60 +115,84 @@ static void
 ephy_rss_feed_subscribe_cb (GtkAction *action,
 			    EphyWindow *window)
 {
-	const GValue *value;
 	GError *error = NULL;
-	EphyEmbedEvent *event;
-	gboolean success;
-	EphyRssExtension *extension = EPHY_RSS_EXTENSION (g_object_get_data (
-												G_OBJECT (window),
-												EPHY_RSS_EXTENSION_DATA_KEY));
+	gboolean success = FALSE;
+	EphyRssExtension *extension;
+	EphyEmbed *embed;
+	WebKitWebView *view;
+	GdkEventButton *event;
+	WebKitHitTestResult *hit_test;
+	char *uri;
 
 	LOG ("Subscribing to the feed");
 
-	event = ephy_window_get_context_event (window);
-	if (event == NULL) return;
+	extension = EPHY_RSS_EXTENSION (g_object_get_data (G_OBJECT (window), EPHY_RSS_EXTENSION_DATA_KEY));
 
-	value = ephy_embed_event_get_property (event, "link");
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+	view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
-	if (!dbus_g_proxy_call (extension->priv->proxy, RSS_DBUS_SUBSCRIBE, &error,
-		G_TYPE_STRING, g_value_get_string (value),
-		G_TYPE_INVALID,
-		G_TYPE_BOOLEAN, &success,
-		G_TYPE_INVALID))
+	event = (GdkEventButton *) (g_object_get_data (G_OBJECT (window),
+				                     "rss-event"));
+
+	hit_test = webkit_web_view_get_hit_test_result (view, event);
+	gdk_event_free ((GdkEvent *) event);
+
+	g_object_get (hit_test, "link-uri", &uri, NULL);
+	g_object_unref (hit_test);
+
+	dbus_g_proxy_call (extension->priv->proxy,
+			   RSS_DBUS_SUBSCRIBE, &error,
+			   G_TYPE_STRING, uri,
+			   G_TYPE_INVALID,
+			   G_TYPE_BOOLEAN, &success,
+			   G_TYPE_INVALID);
+
+	if (!success)
 	{
 		LOG ("Error while retreiving method answer: %s", error->message);
 		g_error_free (error);
 	}
 
-	g_object_set(action, "sensitive", FALSE, "visible", FALSE, NULL);
+	g_object_set (action, "sensitive", FALSE, "visible", FALSE, NULL);
 }
 
 static gboolean
-ephy_rss_ge_context_cb	(EphyEmbed *embed,
-			EphyEmbedEvent *event,
-			EphyWindow *window)
+ephy_rss_button_press_cb (EphyWebView *view,
+			  GdkEventButton *event,
+			  EphyWindow *window)
 {
 	WindowData *data;
-	const GValue *value;
-	const char *address;
 	FeedList *list;
+	WebKitHitTestResult *hit_test;
+	guint context;
+	char *uri;
 	gboolean active = FALSE;
 
-	list = (FeedList *) g_object_get_data (G_OBJECT (embed), FEEDLIST_DATA_KEY);
-	if ((ephy_embed_event_get_context (event) & EPHY_EMBED_CONTEXT_LINK) && (list != NULL))
+	if (event->button != 3 || event->type != GDK_BUTTON_PRESS)
+		return FALSE;
+
+	list = g_object_get_data (G_OBJECT (view), FEEDLIST_DATA_KEY);
+	data = g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
+	g_return_val_if_fail (data != NULL, FALSE);
+
+	hit_test = webkit_web_view_get_hit_test_result (WEBKIT_WEB_VIEW (view), event);
+	g_object_get (hit_test, "context", &context, NULL);
+
+	if ((context & WEBKIT_HIT_TEST_RESULT_CONTEXT_LINK) && (list != NULL))
 	{
-		LOG ("Context menu on a link");
-		data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
-		g_return_val_if_fail (data != NULL, FALSE);
+		g_object_get (hit_test, "link-uri", &uri, NULL);
+		active = rss_feedlist_contains (list, uri);
+		g_free (uri);
 
-		value = ephy_embed_event_get_property (event, "link");
-		address = g_value_get_string (value);
+		g_object_set_data (G_OBJECT (window), "rss-event",
+				   gdk_event_copy ((GdkEvent *) event));
 
-		active = rss_feedlist_contains (list, address);
-
-		LOG ("Showing menu item: %d", active);
-		g_object_set(data->subscribe_action, "sensitive", active, "visible", active, NULL);
+		g_object_set (data->subscribe_action,
+			      "sensitive", active,
+			      "visible", active,
+			      NULL);
 	}
+	g_object_unref (hit_test);
 
 	return FALSE;
 }
@@ -177,6 +202,7 @@ ephy_rss_dialog_display (EphyWindow *window)
 {
 	EphyRssExtensionPrivate *priv;
 	EphyEmbed *embed;
+	EphyWebView *view;
 	FeedList *list;
 	WindowData *data;
 
@@ -187,23 +213,16 @@ ephy_rss_dialog_display (EphyWindow *window)
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
+	view = ephy_embed_get_web_view (embed);
+	g_return_if_fail (view != NULL);
 
-	list = (FeedList *) g_object_get_data (G_OBJECT (embed), FEEDLIST_DATA_KEY);
-	if (list == NULL)
-		return;
+	list = g_object_get_data (G_OBJECT (view), FEEDLIST_DATA_KEY);
+	g_return_if_fail (list != NULL);
 
 	if (priv->dialog == NULL)
 	{
-		RssUI **dialog;
-
 		LOG ("Trying to build dialog");
-
-		priv->dialog = rss_ui_new (list, embed);
-
-		dialog = &priv->dialog;
-
-		g_object_add_weak_pointer (G_OBJECT (priv->dialog),
-					   (gpointer *) dialog);
+		priv->dialog = rss_ui_new (list, embed, data->extension);
 	}
 
 	ephy_dialog_set_parent (EPHY_DIALOG (priv->dialog),
@@ -239,7 +258,7 @@ ephy_rss_update_statusbar (EphyWindow *window,
 	WindowData *data;
 
 	/* Show / Hide statusbar icon */
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
+	data = g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
 	g_return_if_fail (data != NULL);
 
 	g_object_set (data->evbox, "visible", show, NULL);
@@ -253,23 +272,29 @@ ephy_rss_update_action (EphyWindow *window)
 	FeedList *list;
 	gboolean show = TRUE;
 	EphyEmbed *embed;
+	EphyWebView *view;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+	g_return_if_fail (embed != NULL);
+	view = ephy_embed_get_web_view (embed);
+	g_return_if_fail (view != NULL);
 
 	/* The page is loaded, do we have a feed ? */
-	list = (FeedList *) g_object_get_data (G_OBJECT (embed), FEEDLIST_DATA_KEY);
-
+	list = g_object_get_data (G_OBJECT (view), FEEDLIST_DATA_KEY);
 	show = rss_feedlist_length (list) > 0;
 
 	/* Disable the menu item when loading the page */
-	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
+	data = g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
 	g_return_if_fail (data != NULL);
 
 	g_object_set (data->info_action, "sensitive", show, NULL);
 
 	ephy_rss_update_statusbar (window, show);
 
-	g_object_set(data->subscribe_action, "sensitive", show, "visible", show, NULL);
+	g_object_set (data->subscribe_action,
+		      "sensitive", show,
+		      "visible", show,
+		      NULL);
 }
 
 /* Called when the user changes tab */
@@ -289,17 +314,21 @@ impl_attach_tab (EphyExtension *extension,
 		 EphyWindow *window,
 		 EphyEmbed *embed)
 {
+	EphyWebView *view;
+
 	LOG ("Attach rss listener to tab");
 
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
+	view = ephy_embed_get_web_view (embed);
+
 	/* Notify when a new rss feed is parsed */
-	g_signal_connect_after (embed, "new-document-now",
+	g_signal_connect_after (view, "new-document-now",
 				G_CALLBACK (ephy_rss_ge_content_cb), window);
-	g_signal_connect_after (embed, "ge-feed-link",
-			    G_CALLBACK (ephy_rss_ge_feed_cb), window);
-	g_signal_connect (embed, "ge-context-menu",
-			    G_CALLBACK (ephy_rss_ge_context_cb), window);
+	g_signal_connect_after (view, "ge-feed-link",
+				G_CALLBACK (ephy_rss_ge_feed_cb), window);
+	g_signal_connect (view, "button-press-event",
+			  G_CALLBACK (ephy_rss_button_press_cb), window);
 }
 
 /* Stop listening for the detached tab rss feeds */
@@ -308,19 +337,23 @@ impl_detach_tab (EphyExtension *extension,
 		 EphyWindow *window,
 		 EphyEmbed *embed)
 {
+	EphyWebView *view;
+
 	LOG ("Detach tab rss listener");
 
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
+	view = ephy_embed_get_web_view (embed);
+
 	/* We don't want any new rss notif for this tab */
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (ephy_rss_ge_feed_cb), window);
+		(view, G_CALLBACK (ephy_rss_ge_feed_cb), window);
 
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (ephy_rss_ge_content_cb), window);
+		(view, G_CALLBACK (ephy_rss_ge_content_cb), window);
 
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (ephy_rss_ge_context_cb), window);
+		(view, G_CALLBACK (ephy_rss_button_press_cb), window);
 
 	/* destroy data */
 	g_object_set_data (G_OBJECT (embed), FEEDLIST_DATA_KEY, NULL);
@@ -503,13 +536,7 @@ ephy_rss_extension_finalize (GObject *object)
 
 	/* Dispose the dialog */
 	if (extension->priv->dialog != NULL)
-	{
-		RssUI **dialog = &extension->priv->dialog;
-
-		g_object_unref (*dialog);
-		g_object_remove_weak_pointer (G_OBJECT (*dialog),
-					      (gpointer *) dialog);
-	}
+		g_object_unref (extension->priv->dialog);
 
 	parent_class->finalize (object);
 }
@@ -576,3 +603,15 @@ ephy_rss_extension_register_type (GTypeModule *module)
 
 	return type;
 }
+
+RssUI *
+ephy_rss_extension_get_dialog (EphyRssExtension *extension)
+{
+	return extension->priv->dialog;
+}
+
+void
+ephy_rss_extension_set_dialog (EphyRssExtension *extension, RssUI *dialog)
+{
+	extension->priv->dialog = dialog;
+}
